// 
// Copyright © 2019 Samuel Venable
// Copyright © 2014 Seth N. Hetu
// 
// This file is a part of the ENIGMA Development Environment.
// 
// ENIGMA is free software: you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation, version 3 of the license or any later version.
// 
// This application and its source code is distributed AS-IS, WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
// 
// You should have received a copy of the GNU General Public License along
// with this code. If not, see <http://www.gnu.org/licenses/>
//

#include <map>
#include <string>
#include <vector>
#include <istream>

using std::map;
using std::string;
using std::vector;
using std::istream;

namespace ini {

struct IniValue;
struct IniFileSection;

class IniFileIndex {
public:
  IniFileIndex() : commentChar(';') {}

  void load(istream& input, char commentChar);
  void clear();

  bool keyExists(const string& section, const string& key);
  bool sectionExists(const string& section);

  void write(const string& section, const string& key, const string& value);
  void write(const string& section, const string& key, float value);

  string read(const string& section, const string& key, string def) const;
  float read(const string& section, const string& key, float def) const;

  void delKey(const string& section, const string& key);
  void delSection(const string& section);

  bool saveToFile(const string& fname) const;
  string toString() const;

private:
  map<string, IniFileSection> sections;
  vector<string> secNameOrder;
  string postComment;
  char commentChar;
};


struct IniFileSection {
  string prefix;
  map<string, IniValue> props;
  vector<string> propKeyOrder;
};

struct IniValue {
  string value;
  string prefix;
  string postfix;
};

}
