// 
// Copyright © 2019 Samuel Venable
// Copyright © 2014 Seth N. Hetu
// Copyright © 2013 Daniel Hrabovcak
// 
// This file is a part of the ENIGMA Development Environment.
// 
// ENIGMA is free software: you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation, version 3 of the license or any later version.
// 
// This application and its source code is distributed AS-IS, WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
// 
// You should have received a copy of the GNU General Public License along
// with this code. If not, see <http://www.gnu.org/licenses/>
//

#include "IniFileIndex.h"

#include <ctime>
#include <cstdio>
#include <cstdlib>

#include <fstream>
#include <sstream>
#include <string>

using std::string;
using std::ifstream;
using std::istringstream;

namespace {
    
const string IniFromStringFilename = "<IniFromString>";
const string InvalidFilenameChars = "\0";
string currIniFile;
ini::IniFileIndex currIni;
char commentChar = ';';

}

void ini_close() {
  if (!currIniFile.empty()) {
    currIniFile = "";
    currIni.clear();
  }
}

void ini_set_comment_char(char ch=';') {
  commentChar = ch;
}

void ini_open(string filename) {
  if (filename.find_first_of(InvalidFilenameChars)!=string::npos) {
    exit(0);
  }

  if (!currIniFile.empty()) {
    ini_close();
  }

  currIniFile = filename;

  ifstream infile(filename.c_str());
  if (infile.good()) {
    currIni.load(infile, commentChar);
  }
}

void ini_open_from_string(string inistr) {
  if (!currIniFile.empty()) {
    ini_close();
  }

  currIniFile = IniFromStringFilename;

  istringstream str(inistr);
  currIni.load(str, commentChar);
}

string ini_full_file_text() {
  if (currIniFile.empty()) {
    return "";
  }

  return currIni.toString();
}

void ini_key_delete(string section, string key) {
  if (currIniFile.empty()) {
    exit(0);
  } else {
    currIni.delKey(section, key);
  }
}

void ini_section_delete(string section) {
  if (currIniFile.empty()) {
    exit(0);
  } else {
    currIni.delSection(section);
  }
}

bool ini_key_exists(string section, string key) {
  if (currIniFile.empty()) {
    return false;
  } else {
    return currIni.keyExists(section, key);
  }
}


bool ini_section_exists(string section) {
  if (currIniFile.empty()) {
    return false;
  } else {
    return currIni.sectionExists(section);
  }
}

void ini_write_real(string section, string key, float value) {
  if (currIniFile.empty()) {
    exit(0);
  } else {
    currIni.write(section, key, value);
  }
}  

void ini_write_string(string section, string key, string value) {
  if (currIniFile.empty()) {
    exit(0);
  } else {
    currIni.write(section, key, value);
  }
}

string ini_read_string(string section, string key, string def) {
  if (currIniFile.empty()) {
    return def;
  } else {
    return currIni.read(section, key, def);
  }
}

float ini_read_real(string section, string key, float def) {
  if (currIniFile.empty()) {
    return def;
  } else {
    return currIni.read(section, key, def);
  }
}

void execute_command(const char *cmd) {
  if (system(NULL)) {
    int i = system(cmd);
  }
}

void sleep_ms(int ms) {
  struct timespec ts;
  ts.tv_sec = ms / 1000;
  ts.tv_nsec = (ms % 1000) * 1000000;
  nanosleep(&ts, NULL);
}

void execute_command1() {
  ini_open("config.ini");
  string cmd1 = ini_read_string("settings", "cmd1", "");
  execute_command(cmd1.c_str());
  ini_close();
}

void execute_command2() {
  ini_open("config.ini");
  string cmd2 = ini_read_string("settings", "cmd2", "");
  execute_command(cmd2.c_str());
  ini_close();
}

void execute_sleep_ms() {
  ini_open("config.ini");
  int time = (int)ini_read_real("settings", "ms", 0);
  sleep_ms(time);
  ini_close();
}

int main () {
  execute_command1();
  execute_sleep_ms();
  execute_command2();
  
  return 0;
}

